import { useRef, useEffect, useState, useMemo } from 'react';
import { useFrame } from '@react-three/fiber';
import { Sphere, Html, OrbitControls, Line, useGLTF } from '@react-three/drei';
import * as THREE from 'three';
import styles from './Scene.module.css';

interface GeoLocation {
  latitude: number;
  longitude: number;
  city: string;
  country: string;
}

interface ISSPosition {
  latitude: number;
  longitude: number;
  altitude: number;
  velocity: number;
}

// Orbital simulation parameters for fallback
const ISS_ORBITAL_PERIOD = 92.68; // minutes
const ISS_INCLINATION = 51.6; // degrees
const ISS_BASE_ALTITUDE = 408; // km
const ISS_BASE_VELOCITY = 7.66; // km/s

function calculateSimulatedISSPosition(timestamp: number): ISSPosition {
  const minutes = timestamp / 60000; // Convert to minutes
  const orbits = minutes / ISS_ORBITAL_PERIOD;
  const orbitalProgress = (orbits % 1) * 2 * Math.PI;
  
  // Calculate position using simplified orbital mechanics
  const longitude = (orbitalProgress * 180 / Math.PI - 180) % 360;
  const latitude = Math.sin(orbitalProgress) * ISS_INCLINATION;

  return {
    latitude,
    longitude,
    altitude: ISS_BASE_ALTITUDE,
    velocity: ISS_BASE_VELOCITY
  };
}

const companyRegions = [
  {
    name: "United Kingdom",
    coordinates: [51.5074, -0.1278],
    message: "Global Operations Head Office - Leading our worldwide cybersecurity initiatives"
  },
  {
    name: "India",
    coordinates: [20.5937, 78.9629],
    message: "R&D and Technical Support Head Office - Driving innovation and excellence in support"
  },
  {
    name: "Dubai",
    coordinates: [25.2048, 55.2708],
    message: "Middle East Head Office - Pioneering advanced threat protection across MENA"
  }
];

function getSunPosition(timestamp: number) {
  // Convert timestamp to hours
  const totalHours = (timestamp / (1000 * 60 * 60)) % 24;
  
  // Convert time to angle (15 degrees per hour)
  // Subtract 180 degrees to align with Three.js coordinate system
  const longitude = ((totalHours * 15) - 180) * (Math.PI / 180);
  
  // Approximate sun's declination (latitude) based on average tilt
  const latitude = (23.5 * Math.PI / 180); // Earth's axial tilt
  const distance = 5;
  
  const x = distance * Math.cos(latitude) * Math.cos(longitude);
  const y = distance * Math.sin(latitude);
  const z = distance * Math.cos(latitude) * Math.sin(longitude);
  
  return new THREE.Vector3(x, y, z);
}

const PulsingRing = ({ isHovered, radius }: { isHovered: boolean; radius: number }) => {
  const materialRef = useRef<THREE.MeshBasicMaterial>(null);
  
  useFrame((state) => {
    if (materialRef.current) {
      const t = state.clock.getElapsedTime();
      materialRef.current.opacity = (Math.sin(t * 2) * 0.15 + 0.25) * (isHovered ? 1 : 0.5);
    }
  });

  return (
    <mesh>
      <ringGeometry args={[radius, radius + 0.005, 32]} />
      <meshBasicMaterial
        ref={materialRef}
        color="#0000FF"
        transparent
        side={THREE.DoubleSide}
      />
    </mesh>
  );
};

const ElectricLine: React.FC<{
  start: THREE.Vector3;
  end: THREE.Vector3;
  color?: string;
  onAnimationComplete?: () => void;
  delay?: number;
}> = ({ start, end, color = '#4facfe', onAnimationComplete, delay = 0 }) => { 
  const [animationProgress, setAnimationProgress] = useState(0);
  const [animationStarted, setAnimationStarted] = useState(false);
  const startTime = useRef<number | null>(null);
  const duration = 2000; // Reduced duration for snappier animation

  useEffect(() => {
    let timeoutId: NodeJS.Timeout;
    if (delay > 0) {
      timeoutId = setTimeout(() => {
        setAnimationStarted(true);
        startTime.current = Date.now();
      }, delay);
    } else {
      setAnimationStarted(true);
      startTime.current = Date.now();
    }
    return () => {
      if (timeoutId) clearTimeout(timeoutId);
    };
  }, [delay]);

  useFrame(() => {
    if (!animationStarted || startTime.current === null) return;

    const elapsed = Date.now() - startTime.current;
    const progress = Math.min(elapsed / duration, 1);
    const easedProgress = 1 - Math.pow(1 - progress, 3);
    setAnimationProgress(easedProgress);

    if (progress >= 1 && onAnimationComplete) {
      onAnimationComplete();
    }
  });

  const colors = useMemo(() => ({
    primary: new THREE.Color(color),
    glow: new THREE.Color('#ffffff')
  }), [color]);

  const curve = useMemo(() => {
    if (!start || !end) return null;

    const startVec = new THREE.Vector3().copy(start);
    const endVec = new THREE.Vector3().copy(end);
    
    if (startVec.length() === 0 || endVec.length() === 0) return null;
    
    // Calculate a simple arc
    const mid = new THREE.Vector3().addVectors(startVec, endVec).multiplyScalar(0.5);
    const radius = startVec.length();
    mid.normalize().multiplyScalar(radius * 1.2); // Gentle arc
    
    return new THREE.QuadraticBezierCurve3(startVec, mid, endVec);
  }, [start, end]);

  const points = useMemo(() => {
    if (!curve) return [];
    
    try {
      const totalPoints = 100; // Reduced point count for better performance
      const pts = curve.getPoints(totalPoints);
      if (pts.length === 0) return [];
      
      const visiblePoints = Math.max(1, Math.ceil(totalPoints * animationProgress));
      return pts.slice(0, visiblePoints + 1);
    } catch (error) {
      console.error('Error generating points:', error);
      return [];
    }
  }, [curve, animationProgress]);

  if (points.length < 2) return null;

  const pulseIntensity = Math.sin(Date.now() * 0.003) * 0.1 + 0.9; // Subtle pulse

  return (
    <group>
      {/* Main line with glow */}
      <Line
        points={points}
        color={colors.primary}
        lineWidth={3}
        transparent
        opacity={0.8 * pulseIntensity}
      />
      <Line
        points={points}
        color={colors.glow}
        lineWidth={6}
        transparent
        opacity={0.2 * pulseIntensity}
      />
      {/* Endpoint markers */}
      {[points[0], points[points.length - 1]].map((point, index) => (
        <group key={index} position={point}>
          <mesh>
            <sphereGeometry args={[0.03, 16, 16]} />
            <meshBasicMaterial 
              color={colors.primary}
              opacity={pulseIntensity}
              transparent
            />
          </mesh>
          <mesh>
            <ringGeometry args={[0.04, 0.05, 32]} />
            <meshBasicMaterial
              color={colors.primary}
              transparent
              opacity={0.5 * pulseIntensity}
              side={THREE.DoubleSide}
            />
          </mesh>
        </group>
      ))}
    </group>
  );
};

useGLTF.preload('/models/iss/ISS/scene.gltf');

export default function Scene() {
  const isMobile = /iPhone|iPad|iPod|Android/i.test(navigator.userAgent);
  
  // Performance configurations based on device
  const config = useMemo(() => ({
    segments: isMobile ? 32 : 64,
    particleCount: isMobile ? 100 : 200,
    maxMarkers: isMobile ? 5 : 10,
    pixelRatio: isMobile ? Math.min(window.devicePixelRatio, 2) : window.devicePixelRatio
  }), []);

  const sphereRef = useRef<THREE.Mesh<THREE.SphereGeometry>>(null);
  const markerRef = useRef<THREE.Mesh>(null);
  const issRef = useRef<THREE.Group>(null);
  const cloudsRef = useRef<THREE.Mesh>(null);
  const controlsRef = useRef<any>(null);
  const sunLightRef = useRef<THREE.DirectionalLight>(null);
  const [isGlobeLoaded, setIsGlobeLoaded] = useState(false);
  const [showLines, setShowLines] = useState(false);
  const [autoRotate, setAutoRotate] = useState(true);

  // Add a delay after globe load before showing lines
  useEffect(() => {
    if (isGlobeLoaded) {
      const timer = setTimeout(() => {
        setShowLines(true);
      }, 1000); // Use TRANSITION_DURATION instead of hardcoded 1000
      return () => clearTimeout(timer);
    }
  }, [isGlobeLoaded]);

  const [location, setLocation] = useState<GeoLocation | null>(null);
  const [issPosition, setIssPosition] = useState<ISSPosition | null>(null);
  const [showISSInfo, setShowISSInfo] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [hoveredRegion, setHoveredRegion] = useState<typeof companyRegions[0] | null>(null);
  const [selectedRegion, setSelectedRegion] = useState<typeof companyRegions[0] | null>(null);
  const [texturesLoaded, setTexturesLoaded] = useState(false);
  
  // Move texture loading to useEffect to prevent reloading
  const [textures, setTextures] = useState<{
    dayTexture: THREE.Texture;
    nightTexture: THREE.Texture;
    bumpMap: THREE.Texture;
    specularMap: THREE.Texture;
    cloudsTexture: THREE.Texture;
  } | null>(null);

  useEffect(() => {
    const textureLoader = new THREE.TextureLoader();
    const loadTexture = (url: string) => {
      return new Promise<THREE.Texture>((resolve) => {
        textureLoader.load(url, resolve);
      });
    };

    Promise.all([
      loadTexture('https://raw.githubusercontent.com/mrdoob/three.js/dev/examples/textures/planets/earth_atmos_2048.jpg'),
      loadTexture('https://raw.githubusercontent.com/mrdoob/three.js/dev/examples/textures/planets/earth_lights_2048.png'),
      loadTexture('https://raw.githubusercontent.com/mrdoob/three.js/dev/examples/textures/planets/earth_normal_2048.jpg'),
      loadTexture('https://raw.githubusercontent.com/mrdoob/three.js/dev/examples/textures/planets/earth_specular_2048.jpg'),
      loadTexture('https://raw.githubusercontent.com/mrdoob/three.js/dev/examples/textures/planets/earth_clouds_1024.png')
    ]).then(([dayTexture, nightTexture, bumpMap, specularMap, cloudsTexture]) => {
      setTextures({
        dayTexture,
        nightTexture,
        bumpMap,
        specularMap,
        cloudsTexture
      });
      setTexturesLoaded(true);
    });
  }, []);

  const earthShader = {
    uniforms: {
      dayTexture: { value: null },
      nightTexture: { value: null },
      bumpMap: { value: null },
      specularMap: { value: null },
      sunDirection: { value: new THREE.Vector3(1, 0, 0) },
      modelMatrix: { value: new THREE.Matrix4() },
      bumpScale: { value: 0.08 }, 
      nightIntensity: { value: 1.5 },
      dayNightMix: { value: 1.0 },
      terminatorSoftness: { value: 0.5 }, 
      atmosphereStrength: { value: 1.5 } 
    },
    vertexShader: `
      varying vec2 vUv;
      varying vec3 vNormal;
      varying vec3 vWorldPosition;
      
      void main() {
        vUv = uv;
        vNormal = normalize(normalMatrix * normal);
        vec4 worldPosition = modelMatrix * vec4(position, 1.0);
        vWorldPosition = worldPosition.xyz;
        gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
      }
    `,
    fragmentShader: `
      uniform sampler2D dayTexture;
      uniform sampler2D nightTexture;
      uniform sampler2D bumpMap;
      uniform sampler2D specularMap;
      uniform vec3 sunDirection;
      uniform float bumpScale;
      uniform float nightIntensity;
      uniform float dayNightMix;
      uniform float terminatorSoftness;
      uniform float atmosphereStrength;
      
      varying vec2 vUv;
      varying vec3 vNormal;
      varying vec3 vWorldPosition;
      
      void main() {
        // Get textures
        vec4 dayColor = texture2D(dayTexture, vUv);
        vec4 nightColor = texture2D(nightTexture, vUv) * nightIntensity;
        vec4 specularValue = texture2D(specularMap, vUv);
        
        // Normal mapping
        vec3 normalMap = texture2D(bumpMap, vUv).xyz * 2.0 - 1.0;
        normalMap.xy *= bumpScale;
        vec3 worldNormal = normalize(vNormal + normalMap);
        
        // Calculate sun angle
        float sunAngle = dot(worldNormal, normalize(sunDirection));
        
        // Improved terminator calculation with softer transition
        float terminatorWidth = terminatorSoftness;
        float dayMix = smoothstep(-terminatorWidth, terminatorWidth, sunAngle);
        
        // Enhanced atmospheric scattering
        vec3 viewDir = normalize(cameraPosition - vWorldPosition);
        float atmosphericFactor = pow(1.0 - abs(dot(viewDir, worldNormal)), 4.0);
        vec3 atmosphereColor = vec3(0.3, 0.5, 1.0) * atmosphereStrength;
        
        // Specular highlights
        vec3 halfDir = normalize(sunDirection + viewDir);
        float specular = pow(max(dot(worldNormal, halfDir), 0.0), 64.0) * specularValue.r;
        
        // Mix day and night with improved transition
        vec4 color = mix(nightColor, dayColor * 0.85, dayMix); 
        
        // Add atmospheric scattering with adjusted values
        color.rgb += atmosphericFactor * atmosphereColor * (dayMix * 0.3 + 0.2); 
        
        // Add specular highlights only on day side with reduced intensity
        color.rgb += specular * dayMix * vec3(1.0, 0.95, 0.8) * 0.3;
        
        // Enhance contrast while preserving detail
        color.rgb = pow(color.rgb, vec3(1.1)); 
        
        gl_FragColor = color;
      }
    `
  };

  const cloudShader = {
    uniforms: {
      cloudsTexture: { value: null },
      sunDirection: { value: new THREE.Vector3(1, 0, 0) },
      time: { value: 0 },
      opacity: { value: 0.4 } 
    },
    vertexShader: `
      varying vec2 vUv;
      varying vec3 vNormal;
      uniform float time;
      
      void main() {
        vUv = uv;
        vNormal = normalize(normalMatrix * normal);
        
        // Add dynamic cloud movement
        vec3 pos = position;
        float wave = sin(pos.x * 2.0 + time * 0.2) * cos(pos.z * 2.0 + time * 0.1);
        pos.y += wave * 0.005;
        
        gl_Position = projectionMatrix * modelViewMatrix * vec4(pos, 1.0);
      }
    `,
    fragmentShader: `
      uniform sampler2D cloudsTexture;
      uniform vec3 sunDirection;
      uniform float time;
      uniform float opacity;
      
      varying vec2 vUv;
      varying vec3 vNormal;
      
      void main() {
        // Create two UV coordinates for cloud movement
        vec2 uv1 = vUv + vec2(time * 0.0002, time * 0.0001);
        vec2 uv2 = vUv + vec2(-time * 0.00015, -time * 0.00008);
        
        // Sample cloud texture with both UV coordinates
        vec4 clouds1 = texture2D(cloudsTexture, uv1);
        vec4 clouds2 = texture2D(cloudsTexture, uv2);
        
        // Mix the two samples for more dynamic clouds
        vec4 clouds = mix(clouds1, clouds2, 0.5);
        
        // Calculate cloud illumination based on sun direction
        float sunLight = max(dot(vNormal, normalize(sunDirection)), 0.0);
        
        // Adjust cloud brightness based on day/night
        vec4 finalColor = clouds;
        finalColor.a *= opacity;
        finalColor.rgb *= mix(0.2, 1.0, sunLight); 
        
        gl_FragColor = finalColor;
      }
    `
  };

  const [isPanelHovered, setIsPanelHovered] = useState(false);
  const [hoverTimeout, setHoverTimeout] = useState<number | null>(null);

  const handleRegionHover = (region: typeof companyRegions[0] | null, isOutEvent: boolean = false) => {
    if (region && !isOutEvent) {
      setHoveredRegion(region);
      setAutoRotate(false);
    } else {
      if (!selectedRegion) {
        setHoveredRegion(null);
        setAutoRotate(true);
      }
    }
  };

  // Cleanup timeout on unmount
  useEffect(() => {
    return () => {
      if (hoverTimeout) {
        clearTimeout(hoverTimeout);
      }
    };
  }, [hoverTimeout]);

  useEffect(() => {
    const fetchISSPosition = async () => {
      try {
        const response = await fetch('https://api.wheretheiss.at/v1/satellites/25544');
        const data = await response.json();
        
        if (data) {
          setIssPosition({
            latitude: data.latitude,
            longitude: data.longitude,
            altitude: data.altitude,
            velocity: data.velocity
          });
        }
      } catch (err) {
        console.error('Error fetching ISS position, using simulation:', err);
        // Fall back to simulated position
        setIssPosition(calculateSimulatedISSPosition(Date.now()));
      }
    };

    fetchISSPosition();
    const interval = setInterval(fetchISSPosition, 5000);
    return () => clearInterval(interval);
  }, []);

  useEffect(() => {
    const fetchLocation = async () => {
      try {
        const services = [
          {
            url: 'https://get.geojs.io/v1/ip/geo.json',
            handler: async (response: Response) => {
              const data = await response.json();
              return {
                latitude: parseFloat(data.latitude),
                longitude: parseFloat(data.longitude),
                city: data.city,
                country: data.country
              };
            }
          },
          {
            url: 'https://api.ipdata.co/?api-key=test',
            handler: async (response: Response) => {
              const data = await response.json();
              return {
                latitude: data.latitude,
                longitude: data.longitude,
                city: data.city,
                country: data.country_name
              };
            }
          }
        ];

        let locationData = null;
        let lastError = null;

        for (const service of services) {
          try {
            const response = await fetch(service.url);
            if (!response.ok) {
              throw new Error(`HTTP error! status: ${response.status}`);
            }
            locationData = await service.handler(response);
            if (locationData) break;
          } catch (err) {
            lastError = err;
            continue;
          }
        }

        if (locationData) {
          setLocation(locationData);
        } else {
          throw lastError || new Error('Unable to determine location');
        }
      } catch (err) {
        console.error('Location detection failed:', err);
        setError(err instanceof Error ? err.message : 'Location detection failed');
        setLocation({
          latitude: 51.5074,
          longitude: -0.1278,
          city: 'London',
          country: 'United Kingdom'
        });
      }
    };

    fetchLocation();
  }, []);

  useEffect(() => {
    if (location && controlsRef.current) {
      try {
        const phi = (90 - location.latitude) * (Math.PI / 180);
        const theta = (location.longitude + 180) * (Math.PI / 180);
        
        controlsRef.current.object.position.setFromSphericalCoords(
          4,
          phi,
          theta
        );
        
        controlsRef.current.target.set(0, 0, 0);
        controlsRef.current.update();
        
        setTimeout(() => {
          setAutoRotate(true);
        }, 2000);
      } catch (err) {
        console.error('Error positioning camera:', err);
      }
    }
  }, [location]);

  const latLongToVector3 = (lat: number, long: number, radius: number) => {
    const phi = (90 - lat) * (Math.PI / 180);
    const theta = (long + 180) * (Math.PI / 180);

    const x = -(radius * Math.sin(phi) * Math.cos(theta));
    const z = (radius * Math.sin(phi) * Math.sin(theta));
    const y = (radius * Math.cos(phi));

    return new THREE.Vector3(x, y, z);
  };

  useFrame((state) => {
    if (sphereRef.current && sunLightRef.current && textures) {
      const sunPosition = getSunPosition(state.clock.elapsedTime * 1000);
      sunLightRef.current.position.copy(sunPosition);

      // Update the sun direction uniform
      earthShader.uniforms.sunDirection.value = sunPosition.normalize();

      // Calculate dayNightMix based on sun position
      const dotProduct = sunPosition.normalize().dot(new THREE.Vector3(0, 0, 1)); // Assuming globe is oriented with Z-axis as up
      const dayNightMix = Math.max(Math.min(dotProduct * 1.5 + 0.5, 1.0), 0.0);

      earthShader.uniforms.dayNightMix.value = dayNightMix;

      if (controlsRef.current) {
        controlsRef.current.update();
      }

      if (issRef.current && issPosition) {
        // Convert latitude and longitude to radians
        const latitude = issPosition.latitude * (Math.PI / 180);
        const longitude = issPosition.longitude * (Math.PI / 180);

        // Calculate the position of the ISS on the sphere
        const radius = sphereRef.current ? 1 + (issPosition.altitude / 6371) : 1; // Earth radius normalized to 1
        const x = radius * Math.cos(latitude) * Math.cos(longitude);
        const y = radius * Math.sin(latitude);
        const z = radius * Math.cos(latitude) * Math.sin(longitude);

        issRef.current.position.set(x, y, z);
      }
    }

    if (sphereRef.current && sunLightRef.current) {
      const sunPosition = getSunPosition(state.clock.elapsedTime * 1000);
      sunLightRef.current.position.copy(sunPosition);
      
      // Update Earth shader uniforms
      if (sphereRef.current.material instanceof THREE.ShaderMaterial) {
        sphereRef.current.updateMatrixWorld();
        sphereRef.current.material.uniforms.modelMatrix.value.copy(sphereRef.current.matrixWorld);
        sphereRef.current.material.uniforms.sunDirection.value.copy(sunPosition.normalize());
        
        const distance = state.camera.position.length();
        const softness = THREE.MathUtils.lerp(0.1, 0.2, Math.min(distance / 5, 1));
        sphereRef.current.material.uniforms.terminatorSoftness.value = softness;
      }
      
      // Update cloud shader uniforms
      if (cloudsRef.current?.material instanceof THREE.ShaderMaterial) {
        cloudsRef.current.material.uniforms.time.value = state.clock.getElapsedTime();
        cloudsRef.current.material.uniforms.sunDirection.value.copy(sunPosition.normalize());
        
        // Adjust cloud opacity based on camera distance
        const distance = state.camera.position.length();
        const cloudOpacity = THREE.MathUtils.lerp(0.5, 0.3, Math.min(distance / 5, 1));
        cloudsRef.current.material.uniforms.opacity.value = cloudOpacity;
      }

      companyRegions.forEach(region => {
        const position = latLongToVector3(region.coordinates[0], region.coordinates[1], 1.51);
        const marker = sphereRef.current?.getObjectByName(`marker-${region.name}`);
        if (marker && sphereRef.current) {
          marker.position.copy(position);
          marker.setRotationFromEuler(sphereRef.current.rotation);
        }
      });

      if (location && markerRef.current && sphereRef.current) {
        const position = latLongToVector3(location.latitude, location.longitude, 1.51);
        markerRef.current.position.copy(position);
        markerRef.current.setRotationFromEuler(sphereRef.current.rotation);
      }

      // Update ISS position
      if (issRef.current && issPosition) {
        const position = latLongToVector3(issPosition.latitude, issPosition.longitude, 1.65);
        issRef.current.position.copy(position);
        
        // Add a slight wobble to the ISS
        const wobble = Math.sin(state.clock.getElapsedTime() * 2) * 0.02;
        issRef.current.position.y += wobble;
        
        // Rotate the ISS model
        issRef.current.rotation.y = state.clock.getElapsedTime();
      }

      // Update cloud shader time
      if (cloudsRef.current?.material instanceof THREE.ShaderMaterial) {
        cloudsRef.current.material.uniforms.time.value = state.clock.getElapsedTime();
        // Slowly rotate the cloud layer
        cloudsRef.current.rotation.y = state.clock.getElapsedTime() * 0.02;
      }
    }
  });

  const handleControlsEnd = () => {
    setTimeout(() => setAutoRotate(true), 2000); // Give more time before auto-rotation resumes
  };

  return (
    <group
      onClick={() => {
        if (selectedRegion || hoveredRegion) {
          setSelectedRegion(null);
          setHoveredRegion(null);
        }
      }}
    >
      <color attach="background" args={['#000000']} />
      
      <OrbitControls 
        ref={controlsRef}
        enableZoom={false}
        enablePan={false}
        enableDamping={true}
        dampingFactor={0.05}
        rotateSpeed={0.5}
        autoRotate={autoRotate}
        autoRotateSpeed={0.5}
        minPolarAngle={Math.PI / 4}
        maxPolarAngle={Math.PI * 3/4}
        onStart={() => setAutoRotate(false)}
        onEnd={handleControlsEnd}
      />

      <ambientLight intensity={0.8} />
      
      <directionalLight
        ref={sunLightRef}
        position={getSunPosition(Date.now())}
        intensity={3.0}
        color="#FFFFFF"
        castShadow
      />

      <hemisphereLight
        intensity={0.5}
        color="#FFFFFF"
        groundColor="#444444"
      />

      {texturesLoaded && textures && (
        <>
          {/* Earth */}
          <Sphere 
            ref={sphereRef} 
            args={[1, config.segments, config.segments]} 
            scale={1.5}
            onPointerEnter={(e) => {
              e.stopPropagation();
              if (!selectedRegion) {
                setAutoRotate(false);
              }
            }}
            onPointerLeave={(e) => {
              e.stopPropagation();
              if (!selectedRegion && !hoveredRegion) {
                setAutoRotate(true);
              }
            }}
            onAfterRender={() => {
              if (!isGlobeLoaded) {
                setIsGlobeLoaded(true);
              }
            }}
          >
            <shaderMaterial
              attach="material"
              args={[{
                ...earthShader,
                uniforms: {
                  ...earthShader.uniforms,
                  dayTexture: { value: textures.dayTexture },
                  nightTexture: { value: textures.nightTexture },
                  bumpMap: { value: textures.bumpMap },
                  specularMap: { value: textures.specularMap },
                }
              }]}
            />
          </Sphere>

          {/* Atmosphere glow */}
          <Sphere args={[1.02, config.segments, config.segments]} scale={1.5}>
            <meshPhongMaterial
              color="#1a237e"
              transparent
              opacity={0.1}
              depthWrite={false}
              side={THREE.BackSide}
            />
          </Sphere>

          {/* Clouds layer */}
          <Sphere 
            ref={cloudsRef}
            args={[1.015, config.segments, config.segments]} 
            scale={1.5}
          >
            <shaderMaterial
              attach="material"
              args={[{
                ...cloudShader,
                transparent: true,
                uniforms: {
                  ...cloudShader.uniforms,
                  cloudsTexture: { value: textures.cloudsTexture },
                }
              }]}
            />
          </Sphere>
        </>
      )}
      {/* ISS Model */}
      {issPosition && (
        <group
          ref={issRef}
          position={latLongToVector3(issPosition.latitude, issPosition.longitude, 1.65)}
          onPointerEnter={() => setShowISSInfo(true)}
          onPointerLeave={() => setShowISSInfo(false)}
        >
          {(() => {
            const gltf = useGLTF('/models/iss/ISS/scene.gltf');
            return (
              <primitive 
                object={gltf.scene} 
                scale={1}
              />
            );
          })()}
          {/* ISS Trail */}
          <mesh>
            <sphereGeometry args={[0.01, 8, 8]} />
            <meshBasicMaterial
              color="#4169E1"
              transparent
              opacity={0.8}
            />
          </mesh>
          {showISSInfo && (
            <Html
              position={[0, 0.1, 0]}
              center
              style={{
                background: 'rgba(15, 23, 42, 0.9)',
                padding: '12px 16px',
                borderRadius: '8px',
                color: 'white',
                fontSize: '14px',
                whiteSpace: 'nowrap',
                pointerEvents: 'none',
                transition: 'opacity 0.2s ease-in-out',
                opacity: 1,
                boxShadow: '0 4px 6px -1px rgba(0, 0, 0, 0.1), 0 2px 4px -1px rgba(0, 0, 0, 0.06)',
                border: '1px solid rgba(65, 105, 225, 0.3)',
                backdropFilter: 'blur(8px)'
              }}
            >
              <div className="font-semibold mb-1">International Space Station</div>
              <div className="text-blue-200">
                Altitude: {issPosition.altitude} km<br/>
                Speed: {issPosition.velocity.toFixed(2)} km/s<br/>
                Lat: {issPosition.latitude.toFixed(2)}°<br/>
                Long: {issPosition.longitude.toFixed(2)}°
              </div>
            </Html>
          )}
        </group>
      )}

      {/* Region Markers */}
      {isGlobeLoaded && companyRegions.map((region) => {
        // Increase marker height to avoid z-fighting with lines
        const position = latLongToVector3(region.coordinates[0], region.coordinates[1], 1.51);
        return (
          <group 
            key={`marker-${region.name}`}
            position={position}
            onPointerOver={(e) => {
              e.stopPropagation();
              // Prevent event if we're already hovering
              if (hoveredRegion?.name === region.name) return;
              handleRegionHover(region, false);
            }}
            onPointerOut={(e) => {
              e.stopPropagation();
              // Only handle pointer out if we're actually hovering this region
              if (hoveredRegion?.name === region.name) {
                handleRegionHover(null, true);
              }
            }}
            onClick={(e) => {
              e.stopPropagation();
              if (hoverTimeout) {
                clearTimeout(hoverTimeout);
                setHoverTimeout(null);
              }
              setSelectedRegion(region);
              setAutoRotate(false);
            }}
            renderOrder={2} // Ensure markers render on top
          >
            {/* Invisible hit area */}
            <mesh>
              <sphereGeometry args={[0.08, 16, 16]} />
              <meshBasicMaterial 
                transparent
                opacity={0}
                side={THREE.DoubleSide}
                depthTest={false} // Ensure hit area is always detectable
              />
            </mesh>
            {/* Visible marker */}
            <mesh renderOrder={3}>
              <sphereGeometry args={[0.02, 16, 16]} />
              <meshBasicMaterial 
                color="#0000FF"
                transparent
                opacity={hoveredRegion?.name === region.name ? 1 : 0.7}
                depthTest={true}
              />
            </mesh>
            <mesh renderOrder={3}>
              <ringGeometry args={[0.025, 0.03, 32]} />
              <meshBasicMaterial
                color="#0000FF"
                transparent
                opacity={hoveredRegion?.name === region.name ? 0.5 : 0.3}
                side={THREE.DoubleSide}
                depthTest={true}
              />
            </mesh>
            {/* Add pulsing effect when hovered */}
            <PulsingRing 
              isHovered={hoveredRegion?.name === region.name} 
              radius={0.035}
            />
          </group>
        );
      })}

      {/* Electric Lines */}
      {isGlobeLoaded && showLines && companyRegions.map((region) => {
        // Skip the line between India and UK
        if (region.name === "India" && companyRegions[(companyRegions.indexOf(region) + 1) % companyRegions.length].name === "United Kingdom") {
          return null;
        }
        if (region.name === "United Kingdom" && companyRegions[(companyRegions.indexOf(region) + 1) % companyRegions.length].name === "India") {
          return null;
        }
        
        const nextRegion = companyRegions[(companyRegions.indexOf(region) + 1) % companyRegions.length];
        const startPos = latLongToVector3(region.coordinates[0], region.coordinates[1], 1.505);
        const endPos = latLongToVector3(nextRegion.coordinates[0], nextRegion.coordinates[1], 1.505);
        
        // Calculate the actual line index excluding skipped lines
        const activeLineIndex = companyRegions.slice(0, companyRegions.indexOf(region)).filter((r, i) => {
          const nextR = companyRegions[(i + 1) % companyRegions.length];
          return !(
            (r.name === "India" && nextR.name === "United Kingdom") ||
            (r.name === "United Kingdom" && nextR.name === "India")
          );
        }).length;
        
        return (
          <ElectricLine
            key={`line-${region.name}-${nextRegion.name}`}
            start={startPos}
            end={endPos}
            color="#4F6BFF"
            delay={activeLineIndex * 2000} // Wait for previous line to complete
          />
        );
      })}

      {/* Centered Info Panel */}
      {(hoveredRegion || selectedRegion) && (
        <Html
          position={[0, 0, 0]}
          style={{
            width: '100%',
            height: '100%',
            pointerEvents: 'none', 
          }}
        >
          <div 
            className={`${styles.panel} ${isPanelHovered ? styles.panelHovered : ''}`}
            onPointerOver={(e) => {
              e.stopPropagation();
              setIsPanelHovered(true);
            }}
            onPointerOut={(e) => {
              e.stopPropagation();
              setIsPanelHovered(false);
            }}
          >
            <div className={styles.panelContent}>
              <div className={styles.panelHeader}>
                {(hoveredRegion || selectedRegion)?.name}
              </div>
              <div className={styles.panelBody}>
                {(hoveredRegion || selectedRegion)?.message}
              </div>
              <div className={styles.panelFooter}>
                Click anywhere to dismiss
              </div>
            </div>
          </div>
        </Html>
      )}

      {/* Error Notification */}
      {error && (
        <Html position={[0, 2, 0]}>
          <div className={styles.errorNotification}>
            {error}
          </div>
        </Html>
      )}

      {/* User Location Marker */}
      {location && (
        <group>
          <mesh
            ref={markerRef}
            position={latLongToVector3(location.latitude, location.longitude, 1.51)}
            onPointerEnter={() => {
              setAutoRotate(false);
            }}
            onPointerLeave={() => {
              setTimeout(() => setAutoRotate(true), 2000);
            }}
          >
            <sphereGeometry args={[0.035, 16, 16]} />
            <meshBasicMaterial
              color="#2ECC71"
              transparent
              opacity={0.9}
            />
          </mesh>
          {/* Outer glow for user location */}
          <mesh
            position={latLongToVector3(location.latitude, location.longitude, 1.51)}
          >
            <sphereGeometry args={[0.045, 16, 16]} />
            <meshBasicMaterial
              color="#2ECC71"
              transparent
              opacity={0.3}
            />
          </mesh>
          {/* Pulsing effect */}
          <mesh
            position={latLongToVector3(location.latitude, location.longitude, 1.51)}
          >
            <sphereGeometry args={[0.07, 16, 16]} />
            <meshBasicMaterial
              color="#2ECC71"
              transparent
              opacity={Math.abs(Math.sin(Date.now() * 0.003)) * 0.2}
            />
          </mesh>
        </group>
      )}
    </group>
  );
}